package salsa.language;

import salsa.language.exceptions.*;
import salsa.naming.NamingService;
import salsa.messaging.*;

/**
 * The ServiceFactory manufactures references to the standard location dependent
 * services used by an actor. These include the TheaterService,
 * MessagingService, NamingService, and MigrationService.
 * 
 * In the case of a standalone actor, the ServiceFactory will create a theater
 * instance if one is needed. If a theater is already running a reference to
 * will be passed to the requesting actor.
 * 
 * The ServiceFactory is implemented as a synchronized, passive, singleton.
 * 
 * The default services are:<br>
 * <table> 
 * <tr><td>Default: </td><td>System Property: </td></tr>
 * <tr><td>wwc.messaging.Theater</td><td></td>-Dtheater=yourclass</tr>
 * <tr><td>wwc.messaging.Messenger</td><td>-Dmessaging=yourclass</td></tr>
 * <tr><td>wwc.messaging.Migrator</td><td>-Dmigration=yourclass</td></tr>
 * <tr><td>Wwc.naming.WwcNamingImpl</td><td>-Dnaming=yourclass</td></tr>
 * </table>
 * 
 * Classes defined in system properties must have no argument contructors and
 * should implement the their respective service interface.
 * 
 * @author stepha
 */

public class ServiceFactory {
	// Service implementations.
	private static TheaterService theater = null;
	private static MessagingService messaging = null;
	private static NamingService naming = null;
	private static MigrationService migration = null;

	/**
	 * Constructor for Service Factory
	 * This allows for different service implementations
	 * to be specified.
	 */

	public synchronized static void createFactory(TheaterService _theater, MessagingService _messaging, 
						      NamingService _naming, MigrationService _migration) {
		theater = _theater;
		naming = _naming;
		messaging = _messaging;
		migration = _migration;
	}


	/**
	 * Returns the messagingImpl.
	 * @return MessagingService
	 */
	public synchronized static MessagingService getMessaging() {
		if (messaging == null) {
			// Check for the messagingService.
			String className = System.getProperty( "messaging" );
			if (className == null) {
				className = "wwc.messaging.Messenger";
			}

			try {
				messaging = (MessagingService)Class.forName( className ).newInstance();

			} catch (Exception e) {
				throw new SalsaWrapperException( "Exception creating default messaging service.", e );
			}
		}
		
		return messaging;
	}

	/**
	 * Returns the migrationImpl.
	 * @return MigrationService
	 */
	public synchronized static MigrationService getMigration() {
		if (migration == null) {
			// Check for the migration service.
			String className = System.getProperty( "migration" );
			if (className == null) {
				className = "wwc.messaging.Migrator";
			}

			try {
				migration = (MigrationService)Class.forName( className ).newInstance();

			} catch (Exception e) {
				throw new SalsaWrapperException( "Exception creating default migration service.", e );
			}
		}
		
		return migration;
	}

	/**
	 * Returns the namingImpl.
	 * @return NamingService
	 */
	public synchronized static NamingService getNaming() {
		if (naming == null) {
			// Check for the naming service.
			String className = System.getProperty( "naming" );
			NamingService MessagingServiceImpl;
			if (className == null) {
				className = "wwc.naming.WwcNamingImpl";
			}

			try {
				naming = (NamingService)Class.forName( className ).newInstance();

			} catch (Exception e) {
				throw new SalsaWrapperException( "Exception creating default naming service.", e );
			}
		}
			
		return naming;
	}

	/**
	 * Returns the theaterImpl.
	 * @return TheaterService
	 */
	public synchronized static TheaterService getTheater() {
		if (theater == null) {
			String className = System.getProperty( "theater" );
			if (className == null) {
				className = "wwc.messaging.Theater";
			}

			try {
				theater = (TheaterService)Class.forName( className ).newInstance();

			} catch (Exception e) {
				throw new SalsaWrapperException( "Exception creating default theater service.", e );
			}
		}

		return theater;
	}
}
